/*
 * include/linux/nfsd/upcall.h
 *
 * nfsd-userspace upcall stuff.
 *
 * Copyright (C) 2002 Ed V. Bartosh <ed@sam-solutions.net>
 */

#ifndef LINUX_NFSD_UPCALL_H
#define LINUX_NFSD_UPCALL_H

#ifdef __KERNEL__
#include <linux/config.h>
#include <linux/nfsd/const.h>

/* pipe to userspace is needed for these features */
#if defined(CONFIG_NFSD_UGIDMAP) || defined(CONFIG_NFSD_I18N)
#define NFSD_UPCALL	1
#endif

#ifdef  NFSD_UPCALL

#define SIGN_LEN 8

int		nfsd_initpipe(const int fd);
int		pipe_alive(void);
int		to_pipe(void *req, size_t size);

#ifdef  CONFIG_NFSD_UGIDMAP
#define SIGN_UGIDMAP "__UGID"

struct nfsctl_ugidmap;

struct ugidmap_request 
{
        char    sign[SIGN_LEN];
        ulong   ex_ident;
        uid_t   uid;
        gid_t   gid;
	gid_t   groups[NGROUPS];
};

int                     exp_ugidupdate(struct nfsctl_ugidmap *ugidmap);
int                     ugm_cache_init(void);
int			is_registered(void);
struct nfsctl_ugidmap * ugid_map(const struct svc_export *exp, struct svc_cred *cred);

#endif /* CONFIG_NFSD_UGIDMAP */

#ifdef  CONFIG_NFSD_I18N
#define SIGN_I18N "__I18N"

struct nfsctl_i18nmap;

struct i18n_request
{
        char    sign[SIGN_LEN];
	ulong   ex_ident;
        uid_t   uid;
        gid_t   gid;
	gid_t   groups[NGROUPS];
	char	istr[NFS_MAXPATHLEN+1];
	char	ostr[NFS_MAXPATHLEN+1];
	size_t 	ilen;
	size_t 	olen;
	int	back;
};

int  exp_i18nupdate(struct nfsctl_i18nmap *pmap);
int  i18n_cache_init(void);
int i18n_map(struct svc_cred *cred, ulong ex_ident, char **istr, char *ostr, int *len, int back);

#endif /* CONFIG_NFSD_I18N */

#endif /* NFSD_UPCALL */
#endif /* __KERNEL__ */
#endif /* LINUX_NFSD_UPCALL_H */
